/*
 * This file is part of sidplayfp, a console SID player.
 *
 * Copyright 2013-2017 Leandro Nini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef UTILS_H
#define UTILS_H

#include <string>

#include "ini/types.h"

class utils
{
public:
    class error {};

private:
#ifdef _WIN32
    static SID_STRING getPath();
#else
    static SID_STRING getPath(const char* id, const char* def);
#endif

public:
    static SID_STRING getDataPath();
    
    static SID_STRING getConfigPath();

#ifdef _WIN32
    static SID_STRING getExecPath();
#endif
};

#endif
